// -*- C++ -*- (c) 2007, 2008 Petr Rockai <me@mornfall.net>

#include <ept/core/apt/action.h>
#include <ept/core/desktop.h>

#include <adept/packagedata.h>
#include <adept/tokenmodel.h>
#include <adept/packagelist.h>
#include <adept/util.h>

#include <KLineEdit>
#include <KIcon>
#include <KLocale>
#include <KVBox>
#include <KPageWidget>

#ifndef ADEPT_APPLICATIONSWIDGET_H
#define ADEPT_APPLICATIONSWIDGET_H

namespace adept {

using namespace ept::core;
namespace str = wibble::str;

struct AppGroupPolicy : desktop::GroupPolicy {
    QString u8group( const desktop::Entry &e ) {
        /* for ( Range< std::string > i = r; i != i.end(); ++i ) {
            if ( i->find( "X-KDE-settings" ) != std::string::npos )
                return "Settings";
            if ( i->find( "X-KDE-information" ) != std::string::npos )
                return "Settings";
                } */
        if ( e.inCategory( "Development" ) ) return i18n( "Development" );
        if ( !e.inCategory( "Education" ) ) {
            if ( e.inCategory( "Astronomy" )
                 || e.inCategory( "Biology" )
                 || e.inCategory( "Chemistry" )
                 || e.inCategory( "Geology" )
                 || e.inCategory( "MedicalSoftware" )
                 || e.inCategory( "Physics" )
                 || e.inCategory( "Math" )
                 || e.inCategory( "Science" ) )
                return i18n( "Science" );
        } else {
            return i18n( "Edutainment" );
        }
        if ( e.inCategory( "Game" ) ) return i18n( "Games" );
        if ( e.inCategory( "Graphics" ) ) return i18n( "Graphics" );
        if ( e.inCategory( "Network" ) ) return i18n( "Internet" );
        if ( e.inCategory( "AudioVideo" ) ) return i18n( "Multimedia" );
        if ( e.inCategory( "Office" ) ) return i18n( "Office" );
        if ( e.inCategory( "Settings" ) ) return i18n( "Settings" );
        if ( e.inCategory( "System" ) ) return i18n( "System" );
        if ( e.inCategory( "Utility" ) ) {
            if ( e.inCategory( "Accessibility" ) )
                return i18n( "Accessibility" );
            return i18n( "Utilities" );
        }
        return i18n( "Others" );
    }

    std::string group( const desktop::Entry &e ) {
        return s8( u8group( e ) );
    }

    virtual ~AppGroupPolicy() {}
};

struct IconPolicy {
    virtual QString iconForGroup( QString g ) const {
        if ( g == u8( "Settings" ) ) return u8( "preferences-system" );
        if ( g == u8( "Edutainment" ) ) return u8( "applications-accessories" );
        if ( g == u8( "Accessibility" ) ) return u8( "preferences-desktop-accessibility" );
        if ( g == u8( "Others" ) ) return u8( "applications-other" );
        return QString( "applications-" ) + g.toLower();
    }
    virtual ~IconPolicy() {}
};

class ApplicationsPageModel : public KPageModel
{
    Q_OBJECT
public:
    typedef std::map< std::string, PackageListView * > Pages;
    typedef std::pair< std::string, PackageListView * > Pair;
    typedef std::vector< Pair > PageVec;
    IconPolicy iconPol;

protected:
    PageVec pages;
public:
    explicit ApplicationsPageModel( QObject *parent = 0 ) 
        : KPageModel( parent )
    {}

    virtual QModelIndex index( int row, int column,
                               const QModelIndex & = QModelIndex()
        ) const
    {
        return createIndex( row, column, 0 );
    }

    bool valid( const QModelIndex &i ) const {
        return i.row() < static_cast< int >( pages.size() ) && i.column() == 0;
    }

    std::string name( const QModelIndex &i ) {
        if ( !valid( i ) )
            return std::string();
        return pages[ i.row() ].first;
    }

    QVariant data( const QModelIndex &i, int role = Qt::DisplayRole ) const
    {
        if ( !valid( i ) )
            return QVariant();
        Pair p = pages[ i.row() ];
        switch (role) {
            case Qt::DisplayRole:
            case HeaderRole: return QVariant::fromValue( u8( p.first ) );
            case WidgetRole: return QVariant::fromValue(
                static_cast< QWidget * >( p.second ) );
            case Qt::DecorationRole: return QVariant::fromValue(
                QIcon( KIcon( iconPol.iconForGroup( u8( p.first ) ) ) ) );
            default: return QVariant();
        }
    }

    QModelIndex parent( const QModelIndex & ) const {
        return QModelIndex();
    }

    int columnCount( const QModelIndex & ) const {
        return 1;
    }

    int rowCount( const QModelIndex &a ) const {
        if ( a == QModelIndex() )
            return pages.size();
        return 0;
    }

    QModelIndex nameIndex( std::string n ) const {
        int j = 0;
        for ( PageVec::const_iterator i = pages.begin(); i != pages.end(); ++i )
        {
            if ( i->first == n )
                return createIndex( j, 0, 0 );
            ++ j;
        }
        return createIndex( 0, 0, 0 );
    }

    void update( const Pages &visible ) {
        layoutAboutToBeChanged();
        pages.clear();
        std::copy( visible.begin(), visible.end(), std::back_inserter( pages ) );
        dataChanged( createIndex( 0, 0, 0 ),
                     createIndex( pages.size() - 1, 1, 0 ) );
        layoutChanged();
    }
};

class ApplicationsWidget : public KVBox
{
    Q_OBJECT

    KPageView *pg;
    KLineEdit *m_searchLine;
    
    PackageData &d;
    AppGroupPolicy groupPol;
    ApplicationsPageModel *pg_model;
    typedef std::map< std::string, PackageListView * > Pages;
    typedef std::map< std::string, TokenModel * > Models;
    Pages all, visible;
    Models models;
    std::set< ept::Token > m_xapianSet;
    Validator validator;

public:

    struct Matches {
        ApplicationsWidget *m_apps;
        Matches( ApplicationsWidget *a ) : m_apps( a ) {}
        bool operator()( ept::Token t, std::string n ) const {
            std::string m = str::tolower( s8( m_apps->m_searchLine->text() ) );
            std::string name = str::tolower(
                m_apps->d.desk.get< desktop::Name >( t ) );
            n = str::tolower( n );
            if ( n.find( m ) != std::string::npos )
                return true;
            if ( name.find( m ) != std::string::npos )
                return true;
            ept::Token pkg = m_apps->d.desk.get< desktop::Package >( t );
            if ( m_apps->m_xapianSet.count( pkg ) )
                return true;
            return false;
        }
    };

    ept::Token validate( ept::Token t ) {
        ept::Token pkg = d.desk.get< desktop::Package >( t );
        if ( validator.validate( pkg ).valid() )
            return t;
        return ept::Token();
    }

    template< typename L >
    void fillList( L l ) {
        ProcessEvents::restart();
        QApplication::setOverrideCursor( QCursor( Qt::BusyCursor ) );
        visible.clear();
        std::string current, goal;
        pg->setUpdatesEnabled( false );

        goal = pg_model->name( pg->currentPage() );

        for ( Models::iterator i = models.begin(); i != models.end(); ++i )
            i->second->rewind();

        while ( !l.empty() ) {
            ProcessEvents::check( pg );

            ept::Token t = validate( l.head().token() );

            if ( !t.valid() ) {
                l = l.tail();
                continue;
            }

            std::string g = l.head().template get< desktop::Group >();
            l = l.tail();

            if ( models.find( g ) == models.end() ) {
                TokenModel *m = new TokenModel();
                models.insert( std::make_pair( g, m ) );
            }

            if ( all.find( g ) == all.end() ) {
                PackageListView *w = setupList(
                    this, pg, models.find( g )->second, d );
                w->setSortingEnabled( true );
                w->hide();
                std::cerr << "Adding group: " << g << std::endl;
                all.insert( std::make_pair( g, w ) );
            }
            models.find( g )->second->appendToken( t );

            if ( visible.find( g ) == visible.end() ) {
                if ( current.empty() || (!goal.empty() && g == goal) ) {
                    std::cerr << "setting current to " << g << std::endl;
                    current = g;
                }
                visible.insert( visible.begin(), *all.find( g ) );
                updateVisible( current );
            }

        }
        for ( Models::iterator i = models.begin(); i != models.end(); ++i )
            i->second->end();
        pg->setUpdatesEnabled( true );
        QApplication::restoreOverrideCursor();
    }

    void updateVisible( std::string current ) {
        pg_model->update( visible );
        for ( Pages::iterator i = all.begin(); i != all.end(); ++i ) {
            i->second->hide();
        }
        pg->setCurrentPage( pg_model->nameIndex( current ) );
        all.find( current )->second->show(); // show the widget...
    }

    ApplicationsWidget( PackageData &_d, QWidget *p = 0 )
        : KVBox( p ), d( _d ), validator( _d )
    {
        KGlobal::dirs()->addResourceDir(
            "desktopicon",
            u8( "/usr/share/app-install/icons" ) );
        d.desk.setGroupPolicy( &groupPol );
        m_searchLine = new KLineEdit( this );
        m_searchLine->setClearButtonShown( true );
        m_searchLine->setClickMessage( i18n( "Search applications" ) );
        pg = new KPageView( this );
        pg_model = new ApplicationsPageModel( this );
        pg->setModel( pg_model );
        pg->setFaceType( KPageView::List );
        connect( m_searchLine, SIGNAL( returnPressed() ),
                 this, SLOT( search() ) );
    }

Q_SIGNALS:
    void refreshSig();

public Q_SLOTS:
    void search() {
        m_xapianSet.clear();
        list::output(
            list::map(
                list::take( 50, d.xap.query( s8( m_searchLine->text() ) ).results() ),
                std::mem_fun_ref( &xapian::List::token ) ),
            std::inserter( m_xapianSet, m_xapianSet.begin() ) );
        fillList( d.desk.propertyFilter< desktop::ShortDescription >(
                      Matches( this ) ) );
    }

    void delayed() {
        search();
    }

    void refresh() {
        refreshSig();
    }
};

}

#endif
